#include "DeferredShader.h"
#include "glInterface.h"


using namespace std;


namespace CHCDemoEngine
{

void DeferredShader::Render(FrameBufferObject *fbo)
{
	glPushAttrib(GL_VIEWPORT_BIT);
	glViewport(0, 0, fbo->GetWidth(), fbo->GetHeight());

	glDisable(GL_ALPHA_TEST);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	const float offs = 0.5f;
	
	glOrtho(-offs, offs, -offs, offs, 0, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	cgGLEnableProfile(RenderState::sCgFragmentProfile);

	cgGLBindProgram(sCgDeferredProgram);

	cgGLSetTextureParameter(sPositionsTexParam, positionsTex);
	cgGLEnableTextureParameter(sPositionsTexParam);

	cgGLSetTextureParameter(sColorsTexParam, colorsTex);
	cgGLEnableTextureParameter(sColorsTexParam);

	cgGLSetTextureParameter(sNormalsTexParam, normalsTex);
	cgGLEnableTextureParameter(sNormalsTexParam);
	
	glColor3f(1.0f, 1.0f, 1.0f);

	glBegin(GL_QUADS);

	float offs2 = 0.5f;

	glTexCoord2f(0, 0); glVertex3f(-offs2, -offs2, -0.5f);
	glTexCoord2f(1, 0); glVertex3f( offs2, -offs2, -0.5f);
	glTexCoord2f(1, 1); glVertex3f( offs2,  offs2, -0.5f);
	glTexCoord2f(0, 1); glVertex3f(-offs2,  offs2, -0.5f);

	glEnd();

	cgGLDisableTextureParameter(sColorsTexParam);
	cgGLDisableTextureParameter(sPositionsTexParam);
	cgGLDisableTextureParameter(sNormalsTexParam);

	cgGLDisableProfile(RenderState::sCgFragmentProfile);
	
	glEnable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	glPopAttrib();

	PrintGLerror("deferred shading");
}


} // namespace
